'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var HackTheValleyApolloAdaptor = require('./adaptors/HackTheValleyApolloAdaptor');

// Controllers
var GraphController = require('./controllers/GraphController');
var HackerController = require('./controllers/HackerController');
var UserController = require('./controllers/UserController');
var HackerApplicationController = require('./controllers/HackerApplicationController');

var App = function () {
    /**
     * Construct the App instance
     */
    function App() {
        _classCallCheck(this, App);

        this._token = null;

        this.config = {
            endpoints: require('../config/endpoints')
        };

        this._adaptor = new HackTheValleyApolloAdaptor({
            apolloEndpoint: this.getConfiguration().endpoints.apolloEndpoint
        });

        // Initialize controllers
        this._graphController = new GraphController(this);
        this._hackerController = new HackerController(this);
        this._hackerApplicationController = new HackerApplicationController(this);
        this._userController = new UserController(this);

        // Expose routes
        this.Graph = {
            query: this._graphController.query
        };

        this.Hacker = {
            create: this._hackerController.create,
            update: this._hackerController.update,
            createToken: this._hackerController.createToken,
            sendPasswordResetEmail: this._hackerController.sendPasswordResetEmail,
            resetPassword: this._hackerController.resetPassword
        };

        this.HackerApplication = {
            create: this._hackerApplicationController.create,
            updateQuestion: this._hackerApplicationController.updateQuestion,
            submit: this._hackerApplicationController.submit
        };

        this.User = {
            createToken: this._userController.createToken
        };
    }

    /**
     * Get current configuration
     * @returns {{endpoints: ({apolloEndpoint: string}|{apolloEndpoint})}|*}
     */


    _createClass(App, [{
        key: 'getConfiguration',
        value: function getConfiguration() {
            return this.config;
        }

        /**
         * Get current adaptor
         * @returns {HackTheValleyApolloAdaptor}
         */

    }, {
        key: 'getAdaptor',
        value: function getAdaptor() {
            return this._adaptor;
        }

        /**
         * Set authentication token to passed to server
         * @param token
         */

    }, {
        key: 'setAuthenticationToken',
        value: function setAuthenticationToken(token) {
            this._token = token;
            this._adaptor.setAuthenticationToken(token);
        }

        /**
         * Get current authentication token
         * @returns {null|string}
         */

    }, {
        key: 'getAuthenticationToken',
        value: function getAuthenticationToken() {
            return this._token;
        }
    }]);

    return App;
}();

module.exports = App;