'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Class representing a simple Apollo server link
 * Author: Jun Zheng
 * License: MIT
 */

var SynchronousLink = require('./SynchronousLink');

// apollo-client stuff
var ApolloClient = require('apollo-client').default;
var ApolloInMemoryCache = require('apollo-cache-inmemory').InMemoryCache;
var ApolloHttpLink = require('apollo-link-http').HttpLink;
var gql = require('graphql-tag');
var ApolloLinkContext = require('apollo-link-context');

// Node.js does not support ES6 natively
require('cross-fetch/polyfill');

var ApolloLink = function (_SynchronousLink) {
    _inherits(ApolloLink, _SynchronousLink);

    /**
     * Construct a new ApolloLink
     * @param {string} url
     * @param {object} config = {}
     */
    function ApolloLink(url) {
        var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

        _classCallCheck(this, ApolloLink);

        var _this = _possibleConstructorReturn(this, (ApolloLink.__proto__ || Object.getPrototypeOf(ApolloLink)).call(this, url));

        _this._headers = {};

        var cache = new ApolloInMemoryCache();

        var link = new ApolloHttpLink({
            uri: url,
            fetch: fetch
        });

        if (config.headers) {
            var headersLink = ApolloLinkContext.setContext(function () {
                return {
                    headers: config.headers
                };
            });
            link = headersLink.concat(link);
            _this._headers = config.headers;
        }

        _this._client = new ApolloClient({
            link: link,
            cache: cache,
            defaultOptions: {
                query: {
                    fetchPolicy: 'network-only'
                }
            }
        });
        _this._link = link;
        return _this;
    }

    /**
     * Alias for run but automatically sets type as query
     * @param config
     * @returns {Promise<Object>}
     */


    _createClass(ApolloLink, [{
        key: 'query',
        value: function query(config) {
            return this.run(Object.assign(config, { type: 'query' }));
        }

        /**
         * Alias for run, but automatically sets type as mutation
         * @param config
         * @returns {Promise<Object>}
         */

    }, {
        key: 'mutate',
        value: function mutate(config) {
            return this.run(Object.assign(config, { type: 'mutation' }));
        }

        /**
         * Run a new apollo request
         * @param config
         * @returns {Promise<object>}
         */

    }, {
        key: 'run',
        value: function run(config) {
            var _this2 = this;

            return new Promise(function (resolve, reject) {
                var query = gql(config.query);
                // If the type is query, we call the query
                if (config.type === 'query') {
                    _this2._client.query({
                        query: query,
                        variables: config.variables ? config.variables : {}
                    }).then(function (data) {
                        resolve(data.data);
                    }).catch(function (e) {
                        reject(e);
                    });
                    // Otherwise, we call the mutation
                } else {
                    _this2._client.mutate({
                        mutation: query,
                        variables: config.variables ? config.variables : {}
                    }).then(function (data) {
                        resolve(data.data);
                    }).catch(function (e) {
                        reject(e);
                    });
                }
            });
        }
    }]);

    return ApolloLink;
}(SynchronousLink);

module.exports = ApolloLink;